import { n as nanoid, a as spyOn, f as fn, s as spies } from './jest-mock-8498c46d.js';
import { n as noop } from './utils-d97bd6d9.js';
import { c as commonjsGlobal, g as getDefaultExportFromCjs } from './_commonjsHelpers-c9e3b764.js';

var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
function createChainable(keys, fn) {
  function create(obj) {
    const chain2 = function(...args) {
      return fn.apply(obj, args);
    };
    for (const key of keys) {
      Object.defineProperty(chain2, key, {
        get() {
          return create(__spreadProps(__spreadValues({}, obj), { [key]: true }));
        }
      });
    }
    return chain2;
  }
  const chain = create({});
  chain.fn = fn;
  return chain;
}

const context = {
  tasks: [],
  currentSuite: null
};
function collectTask(task) {
  var _a;
  (_a = context.currentSuite) == null ? void 0 : _a.tasks.push(task);
}
async function runWithSuite(suite, fn) {
  const prev = context.currentSuite;
  context.currentSuite = suite;
  await fn();
  context.currentSuite = prev;
}
function getDefaultTestTimeout() {
  return process.__vitest_worker__.config.testTimeout;
}
function getDefaultHookTimeout() {
  return process.__vitest_worker__.config.hookTimeout;
}
function withTimeout(fn, _timeout) {
  const timeout = _timeout ?? getDefaultTestTimeout();
  if (timeout <= 0 || timeout === Infinity)
    return fn;
  return (...args) => {
    return Promise.race([fn(...args), new Promise((resolve, reject) => {
      const timer = setTimeout(() => {
        clearTimeout(timer);
        reject(new Error(`Test timed out in ${timeout}ms.`));
      }, timeout);
      timer.unref();
    })]);
  };
}
function ensureAsyncTest(fn) {
  if (!fn.length)
    return fn;
  return () => new Promise((resolve, reject) => {
    const done = (...args) => args[0] ? reject(args[0]) : resolve();
    fn(done);
  });
}
function normalizeTest(fn, timeout) {
  return withTimeout(ensureAsyncTest(fn), timeout);
}

const fnMap = /* @__PURE__ */ new WeakMap();
const hooksMap = /* @__PURE__ */ new WeakMap();
function setFn(key, fn) {
  fnMap.set(key, fn);
}
function getFn(key) {
  return fnMap.get(key);
}
function setHooks(key, hooks) {
  hooksMap.set(key, hooks);
}
function getHooks(key) {
  return hooksMap.get(key);
}

const suite = createSuite();
const test = createChainable(["concurrent", "skip", "only", "todo", "fails"], function(name, fn, timeout) {
  getCurrentSuite().test.fn.call(this, name, fn, timeout);
});
const describe = suite;
const it = test;
const defaultSuite = suite("");
function clearContext() {
  context.tasks.length = 0;
  defaultSuite.clear();
  context.currentSuite = defaultSuite;
}
function getCurrentSuite() {
  return context.currentSuite || defaultSuite;
}
function createSuiteHooks() {
  return {
    beforeAll: [],
    afterAll: [],
    beforeEach: [],
    afterEach: []
  };
}
function createSuiteCollector(name, factory = () => {
}, mode, suiteComputeMode) {
  const tasks = [];
  const factoryQueue = [];
  let suite2;
  initSuite();
  const test2 = createChainable(["concurrent", "skip", "only", "todo", "fails"], function(name2, fn, timeout) {
    const mode2 = this.only ? "only" : this.skip ? "skip" : this.todo ? "todo" : "run";
    const computeMode = this.concurrent ? "concurrent" : void 0;
    const test3 = {
      id: nanoid(),
      type: "test",
      name: name2,
      mode: mode2,
      computeMode: computeMode ?? (suiteComputeMode ?? "serial"),
      suite: void 0,
      fails: this.fails
    };
    setFn(test3, normalizeTest(fn || noop, timeout));
    tasks.push(test3);
  });
  const collector = {
    type: "collector",
    name,
    mode,
    test: test2,
    tasks,
    collect,
    clear,
    on: addHook
  };
  function addHook(name2, ...fn) {
    getHooks(suite2)[name2].push(...fn);
  }
  function initSuite() {
    suite2 = {
      id: nanoid(),
      type: "suite",
      computeMode: "serial",
      name,
      mode,
      tasks: []
    };
    setHooks(suite2, createSuiteHooks());
  }
  function clear() {
    tasks.length = 0;
    factoryQueue.length = 0;
    initSuite();
  }
  async function collect(file) {
    factoryQueue.length = 0;
    if (factory)
      await runWithSuite(collector, () => factory(test2));
    const allChildren = await Promise.all([...factoryQueue, ...tasks].map((i) => i.type === "collector" ? i.collect(file) : i));
    suite2.file = file;
    suite2.tasks = allChildren;
    allChildren.forEach((task) => {
      task.suite = suite2;
      if (file)
        task.file = file;
    });
    return suite2;
  }
  collectTask(collector);
  return collector;
}
function createSuite() {
  return createChainable(["concurrent", "skip", "only", "todo"], function(name, factory) {
    const mode = this.only ? "only" : this.skip ? "skip" : this.todo ? "todo" : "run";
    const computeMode = this.concurrent ? "concurrent" : void 0;
    return createSuiteCollector(name, factory, mode, computeMode);
  });
}

var mockdate$1 = {exports: {}};

(function (module, exports) {
(function (global, factory) {
    factory(exports) ;
}(commonjsGlobal, (function (exports) {
    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation.

    Permission to use, copy, modify, and/or distribute this software for any
    purpose with or without fee is hereby granted.

    THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
    PERFORMANCE OF THIS SOFTWARE.
    ***************************************************************************** */
    /* global Reflect, Promise */

    var extendStatics = function(d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };

    function __extends(d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    }

    var RealDate = Date;
    var now = null;
    var MockDate = /** @class */ (function (_super) {
        __extends(Date, _super);
        function Date(y, m, d, h, M, s, ms) {
            _super.call(this) || this;
            var date;
            switch (arguments.length) {
                case 0:
                    if (now !== null) {
                        date = new RealDate(now.valueOf());
                    }
                    else {
                        date = new RealDate();
                    }
                    break;
                case 1:
                    date = new RealDate(y);
                    break;
                default:
                    d = typeof d === 'undefined' ? 1 : d;
                    h = h || 0;
                    M = M || 0;
                    s = s || 0;
                    ms = ms || 0;
                    date = new RealDate(y, m, d, h, M, s, ms);
                    break;
            }
            return date;
        }
        return Date;
    }(RealDate));
    MockDate.prototype = RealDate.prototype;
    MockDate.UTC = RealDate.UTC;
    MockDate.now = function () {
        return new MockDate().valueOf();
    };
    MockDate.parse = function (dateString) {
        return RealDate.parse(dateString);
    };
    MockDate.toString = function () {
        return RealDate.toString();
    };
    function set(date) {
        var dateObj = new Date(date.valueOf());
        if (isNaN(dateObj.getTime())) {
            throw new TypeError('mockdate: The time set is an invalid date: ' + date);
        }
        // @ts-ignore
        Date = MockDate;
        now = dateObj.valueOf();
    }
    function reset() {
        Date = RealDate;
    }
    var mockdate = {
        set: set,
        reset: reset,
    };

    exports.default = mockdate;
    exports.reset = reset;
    exports.set = set;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
}(mockdate$1, mockdate$1.exports));

var mockdate = /*@__PURE__*/getDefaultExportFromCjs(mockdate$1.exports);

const originalSetTimeout = global.setTimeout;
const originalSetInterval = global.setInterval;
const originalClearTimeout = global.clearTimeout;
const originalClearInterval = global.clearInterval;
const MAX_LOOPS = 1e4;
const assertEvery = (assertions, message) => {
  if (assertions.some((a) => !a))
    throw new Error(message);
};
const assertMaxLoop = (times) => {
  if (times >= MAX_LOOPS)
    throw new Error("setTimeout/setInterval called 10 000 times. It's possible it stuck in an infinite loop.");
};
const getNodeTimeout = (id) => {
  const timer = {
    ref: () => timer,
    unref: () => timer,
    hasRef: () => true,
    refresh: () => timer,
    [Symbol.toPrimitive]: () => id
  };
  return timer;
};
class FakeTimers {
  constructor() {
    this._advancedTime = 0;
    this._nestedTime = {};
    this._scopeId = 0;
    this._isNested = false;
    this._isOnlyPending = false;
    this._spyid = 0;
    this._isMocked = false;
    this._tasksQueue = [];
    this._queueCount = 0;
  }
  useFakeTimers() {
    this._isMocked = true;
    this.reset();
    const spyFactory = (spyType, resultBuilder) => {
      return (cb, ms = 0) => {
        const id = ++this._spyid;
        const nestedTo = Object.entries(this._nestedTime).filter(([key]) => Number(key) <= this._scopeId);
        const nestedMs = nestedTo.reduce((total, [, ms2]) => total + ms2, ms);
        const call = { id, cb, ms, nestedMs, scopeId: this._scopeId };
        const task = { type: spyType, call, nested: this._isNested };
        this.pushTask(task);
        return resultBuilder(id, cb);
      };
    };
    this._setTimeout = spyOn(global, "setTimeout").mockImplementation(spyFactory("timeout" /* Timeout */, getNodeTimeout));
    this._setInterval = spyOn(global, "setInterval").mockImplementation(spyFactory("interval" /* Interval */, getNodeTimeout));
    const clearTimerFactory = (spyType) => (id) => {
      if (id === void 0)
        return;
      const index = this._tasksQueue.findIndex(({ call, type }) => type === spyType && call.id === Number(id));
      if (index !== -1)
        this._tasksQueue.splice(index, 1);
    };
    this._clearTimeout = spyOn(global, "clearTimeout").mockImplementation(clearTimerFactory("timeout" /* Timeout */));
    this._clearInterval = spyOn(global, "clearInterval").mockImplementation(clearTimerFactory("interval" /* Interval */));
  }
  useRealTimers() {
    this._isMocked = false;
    this.reset();
    global.setTimeout = originalSetTimeout;
    global.setInterval = originalSetInterval;
    global.clearTimeout = originalClearTimeout;
    global.clearInterval = originalClearInterval;
  }
  runOnlyPendingTimers() {
    this.assertMocked();
    this._isOnlyPending = true;
    this.runQueue();
  }
  runAllTimers() {
    this.assertMocked();
    this.runQueue();
  }
  advanceTimersByTime(ms) {
    this.assertMocked();
    this._advancedTime += ms;
    this.runQueue();
  }
  advanceTimersToNextTimer() {
    this.assertMocked();
    this.callQueueItem(0);
  }
  getTimerCount() {
    this.assertMocked();
    return this._tasksQueue.length;
  }
  reset() {
    var _a, _b, _c, _d;
    this._advancedTime = 0;
    this._nestedTime = {};
    this._isNested = false;
    this._isOnlyPending = false;
    this._spyid = 0;
    this._queueCount = 0;
    this._tasksQueue = [];
    (_a = this._clearInterval) == null ? void 0 : _a.mockRestore();
    (_b = this._clearTimeout) == null ? void 0 : _b.mockRestore();
    (_c = this._setInterval) == null ? void 0 : _c.mockRestore();
    (_d = this._setTimeout) == null ? void 0 : _d.mockRestore();
  }
  callQueueItem(index) {
    var _a, _b;
    const task = this._tasksQueue[index];
    if (!task)
      return;
    const { call, type } = task;
    this._scopeId = call.id;
    this._isNested = true;
    (_a = this._nestedTime)[_b = call.id] ?? (_a[_b] = 0);
    this._nestedTime[call.id] += call.ms;
    if (type === "timeout") {
      this.removeTask(index);
    } else if (type === "interval") {
      call.nestedMs += call.ms;
      const nestedMs = call.nestedMs;
      const closestTask = this._tasksQueue.findIndex(({ type: type2, call: call2 }) => type2 === "interval" && call2.nestedMs < nestedMs);
      if (closestTask !== -1 && closestTask !== index)
        this.ensureQueueOrder();
    }
    call.cb();
    this._queueCount++;
  }
  runQueue() {
    let index = 0;
    while (this._tasksQueue[index]) {
      assertMaxLoop(this._queueCount);
      const { call, nested } = this._tasksQueue[index];
      if (this._advancedTime && call.nestedMs > this._advancedTime)
        break;
      if (this._isOnlyPending && nested) {
        index++;
        continue;
      }
      this.callQueueItem(index);
    }
  }
  removeTask(index) {
    if (index === 0)
      this._tasksQueue.shift();
    else
      this._tasksQueue.splice(index, 1);
  }
  pushTask(task) {
    this._tasksQueue.push(task);
    this.ensureQueueOrder();
  }
  ensureQueueOrder() {
    this._tasksQueue.sort((t1, t2) => {
      const diff = t1.call.nestedMs - t2.call.nestedMs;
      if (diff === 0) {
        if (t1.type === "immediate" /* Immediate */ && t2.type !== "immediate" /* Immediate */)
          return 1;
        return 0;
      }
      return diff;
    });
  }
  assertMocked() {
    assertEvery([
      this._isMocked,
      this._setTimeout,
      this._setInterval,
      this._clearTimeout,
      this._clearInterval
    ], 'timers are not mocked. try calling "vitest.useFakeTimers()" first');
  }
}

class VitestUtils {
  constructor() {
    this.spyOn = spyOn;
    this.fn = fn;
    this._timers = new FakeTimers();
    this._mockedDate = null;
  }
  useFakeTimers() {
    return this._timers.useFakeTimers();
  }
  useRealTimers() {
    return this._timers.useRealTimers();
  }
  runOnlyPendingTimers() {
    return this._timers.runOnlyPendingTimers();
  }
  runAllTimers() {
    return this._timers.runAllTimers();
  }
  advanceTimersByTime(ms) {
    return this._timers.advanceTimersByTime(ms);
  }
  advanceTimersToNextTimer() {
    return this._timers.advanceTimersToNextTimer();
  }
  getTimerCount() {
    return this._timers.getTimerCount();
  }
  mockCurrentDate(date) {
    this._mockedDate = date;
    mockdate.set(date);
  }
  restoreCurrentDate() {
    this._mockedDate = null;
    mockdate.reset();
  }
  getMockedDate() {
    return this._mockedDate;
  }
  mock(path) {
  }
  unmock(path) {
  }
  async importActual(path) {
    return {};
  }
  async importMock(path) {
    return {};
  }
  mocked(item, _deep = false) {
    return item;
  }
  isMockFunction(fn2) {
    return typeof fn2 === "function" && "__isSpy" in fn2 && fn2.__isSpy;
  }
  clearAllMocks() {
    __vitest__clearMocks__({ clearMocks: true });
    spies.forEach((spy) => spy.mockClear());
    return this;
  }
  resetAllMocks() {
    __vitest__clearMocks__({ mockReset: true });
    spies.forEach((spy) => spy.mockReset());
    return this;
  }
  restoreAllMocks() {
    __vitest__clearMocks__({ restoreMocks: true });
    spies.forEach((spy) => spy.mockRestore());
    return this;
  }
}
const vitest = new VitestUtils();
const vi = vitest;

export { getDefaultHookTimeout as a, vi as b, clearContext as c, describe as d, defaultSuite as e, setHooks as f, getCurrentSuite as g, getHooks as h, it as i, context as j, getFn as k, suite as s, test as t, vitest as v, withTimeout as w };
